# coding: utf-8
<%inherit file="master.html" />

<%def name="primary()">
	<h2>Type Anything</h2>

	<form id="input" method="post" class="container">
		<textarea id="snippet" name="snippet">${ q }</textarea>
		<input type="submit" id="go" value="Submit" />
		<p id="multi-line">For multiple lines, press Shift + Enter</p>
	</form>
</%def>

<%def name="secondary()">
	% if results:
	<div id="result">

		<div id="api-result">
			<h3>Structured Response</h3>
			<pre>${ json_results | h }</pre>
		</div>

	</div>
	% endif
	<div id="examples">
		<h2>&#8681; Try These &#8681;</h2>

		<div class="container">
			<div class="example names">
				<h3>Names</h3>
				<ul>
					<li><a href="#">Mr Ryan G. Funnel Esq</a></li>
					<li><a href="#">Jordan Headlock Jr</a></li>
					<li><a href="#">Cpl Ryan George Steve Buble</a></li>
					<li><a href="#">Pres Barack Q. Obama</a></li>
				</ul>
			</div>

			<div class="example dates">
				<h3>Dates</h3>
				<ul>
					<li><a href="#">March 31st, 2005</a></li>
					<li><a href="#">Last Year</a></li>
					<li><a href="#">300 seconds before now</a></li>
					<li><a href="#">2015-03-20 plus 30 days</a></li>
				</ul>
			</div>

			<div class="example equations">
				<h3>Math Equations</h3>
				<ul>
					<li><a href="#">5 x 5</a></li>
					<li><a href="#">The square root of 169</a></li>
					<li><a href="#">2 + 2</a></li>
					<li><a href="#">(3^2(5+24))-3</a></li>
				</ul>
			</div>
		</div>

		<div class="container">
			<div class="example urls">
				<h3>URLs and Emails</h3>
				<ul>
					<li><a href="#">http://www.google.com/</a></li>
					<li><a href="#">youtube.com</a></li>
					<li><a href="#">random@email.com</a></li>
					<li><a href="#">192.168.1.1</a></li>
				</ul>
			</div>

			<div class="example phone">
				<h3>Phone Numbers</h3>
				<ul>
					<li><a href="#">1-800-555-2582</a></li>
					<li><a href="#">+44 (013) 33-44-122</a></li>
					<li><a href="#">1 (650) 253-0000</a></li>
					<li><a href="#">713-439-7086</a></li>
				</ul>
			</div>

			<div class="example numbers">
				<h3>Other Numbers</h3>
				<ul>
					<li><a href="#">-1,884.05</a></li>
					<li><a href="#">MMDCXLVI</a></li>
					<li><a href="#">0xDEADBEEF</a></li>
					<li><a href="#">Five Thousand, Three Hundred Twenty</a></li>
				</ul>
			</div>

		</div>

		<div class="container">
			<div class="example landmarks">
				<h3>Landmarks</h3>
				<ul>
					<li><a href="#">Yellowstone</a></li>
					<li><a href="#">The Lincoln Memorial</a></li>
					<li><a href="#">Grand Canyon</a></li>
					<li><a href="#">Mount Rushmore</a></li>
				</ul>
			</div>

			<div class="example measurement">
				<h3>Measurement</h3>
				<ul>
					<li><a href="#">900 &deg;C</a></li>
					<li><a href="#">54 square miles</a></li>
					<li><a href="#">12 parsecs</a></li>
					<li><a href="#">400 gallons of gas</a></li>
				</ul>
			</div>

			<div class="example programming">
				<h3>Programming/Code</h3>
				<ul>
					<li><a href="#">class MyClass { public $prop1 = "I'm a class property!"; } $obj = new MyClass;</a></li>
					<li><a href="#">if len(myArr) in range(2,4): self.myStuff += (5 * len(myArr))</a></li>
					<li><a href="#">def inherit c = Object, *a, &b c = c.class unless Class === c</a></li>
                    <li><a href="#">cout << setprecision(10) << ldn.dx << " = 0x" << hex << ldn.lx[2] << ldn.lx[1] << ldn.lx[0] << endl;</a></li>
				</ul>
			</div>
		</div>

		<div class="container">
			<div class="example coordinates">
				<h3>Coordinates</h3>
				<ul>
					<li><a href="#">-23.5234, 56.7286</a></li>
					<li><a href="#">40° 26' 46.56" N 79° 58' 56.88" W</a></li>
					<li><a href="#">13° 34.425' S 45° 37.983' W</a></li>
					<li><a href="#">40.244° N 79.123° E</a></li>
				</ul>
			</div>

			<div class="example postalcodes">
				<h3>Postal Codes</h3>
				<ul>
					<li><a href="#">77002-2904</a></li>
					<li><a href="#">90210</a></li>
					<li><a href="#">75778 CEDEX 16</a></li>
					<li><a href="#">80219</a></li>
				</ul>
			</div>

			<div class="example addresses">
				<h3>Addresses</h3>
				<ul>
					<li><a href="#">1600 Pennsylvania Avenue, Washington, DC</a></li>
					<li><a href="#">1 Infinite Loop, Cupertino, CA 95014</a></li>
					<li><a href="#">1600 Amphitheatre Parkway, Mountain View, CA 94043</a></li>
					<li><a href="#">5200 Buffington Road, Atlanta, GA 30349-2998</a></li>
				</ul>
			</div>
		</div>
	</div>
</%def>

<%def name="footer()">
    <div class="ss_copyright"><a href="https://github.com/SerenitySoftwareLLC/cahoots">Cahoots</a> is Copyright © 2012-2015 <a href="https://github.com/SerenitySoftwareLLC">Serenity Software</a>, LLC and <a href="https://raw.githubusercontent.com/SerenitySoftwareLLC/cahoots/master/LICENSE">MIT</a> licensed.</div>
    <div class="favicon_copyright">Favicon made by <a href="http://www.flaticon.com/authors/freepik" title="Freepik">Freepik</a> from <a href="http://www.flaticon.com" title="Flaticon">www.flaticon.com</a> is licensed by <a href="http://creativecommons.org/licenses/by/3.0/" title="Creative Commons BY 3.0">CC BY 3.0</a>.</div>
    <div class="background_copyright">Background made by <a href="http://www.atlemo.com/" title="Atle Mo.">Atle Mo.</a> from <a href="http://subtlepatterns.com/" title="Flaticon">subtlepatterns.com/</a> is licensed by <a href="http://creativecommons.org/licenses/by/3.0/" title="Creative Commons BY 3.0">CC BY 3.0</a>.</div>
</%def>