html, body {
	font-family: Helvetica, Tahoma, Arial, sans-serif;
	background-color: #EDEDED;
	margin: 0;
	padding: 0;
	height: 100%;
}

body {
	font-size: 12px;
}

h1, h2, h3, h4, h5, h6 {
	font-family: Oswald, Helvetica, Tahoma, Arial, sans-serif;
	color: #444;
	line-height: 1.2;
	margin: .5em 0;
	padding: .5em 0;
}

h1 {
	font-size: 28px;
}

h2 {
	font-size: 24px;
}

h3 {
	font-size: 18px;
}

h4 {
	font-size: 14px;
}

h5 {
	font-size: 12px;
}

h6 {
	font-size: 8px;
}

a:link, a:visited {
	color: #072d66;
}

a:hover, a:active {
	color: #50adbf;
}

.container:before,
.container:after {
    content:"";
    display:table;
}
.container:after {
    clear:both;
}
/* For IE 6/7 (trigger hasLayout) */
.container {
    zoom:1;
    width: 100%;
}

.container-content {
	margin: 0 auto;
	width: 800px;
}

input.bordered,
textarea.bordered {
	border-color: #999;
}

#header {
    background: url('/static/dark_leather.png') top left repeat;
    padding: 16px 0 0 0;
}

#header h1 {
	clear: none;
	color: #DEDEDE;
	text-align: left;
	font-size: 32px;
	margin: 0;
	padding: 0;
	display: inline-block;
}

#header h1 a {
	text-decoration: none;
	color: #DEDEDE;
}

#header h1 a:hover {
	color: #5bc4d9;
}

#header h2 {
	clear: none;
	color: #DEDEDE;
	text-align: right;
	font-size: 18px;
	font-weight: normal;
	margin: 0 0 0 5px;
	padding: 0;
	display: inline-block;
}

#primary {
  height: 282px;
  padding-top: 65px;
}

#primary h2 {
	color: #5bc4d9;
	font-weight: normal;
	margin: 0 0 10px;
	padding: 0;
}

#input {
	margin: 0;
	padding: 0;
	width: 100%;
}

#input #snippet {
	color: #444;
	width: 788px;
	height: 120px;
	padding: 5px;
	font-weight: 400;
	font-size: 16px;
	font-family: Oswald, Helvetica, Tahoma, Arial, sans-serif;
}

#input #go {
	background-color: #EDEDED;
	clear: none;
	float: left;
	font-size: 16px;
	font-weight: bold;
	color: #333;
	font-family: Oswald, Helvetica, Tahoma, Arial, sans-serif;
	display: block;
	margin-top: 10px;
	cursor: pointer;
	padding: 5px 10px;
}

#input #go:hover {
	cursor: pointer;
	color: #111;
}

#multi-line {
	clear: none;
	float: right;
	color: #EDEDED;
	font-size: 11px;
	font-weight: bold;
	margin: 10px 12px 0 0;
}


#secondary {
	border-top: 5px solid #50adbf;
	padding-top: 15px;
}

#examples h2 {
	text-align: center;
	margin: 10px 0;
	padding: 0;
}

#examples .example {
	float: left;
	width: 33%;
	overflow: hidden;
}

#examples .example ul {
	margin: 10px 0px 10px 15px;
	padding: 0;
}

#examples .example ul li {
	margin: 4px 0;
	padding: 0;
}

#examples .example ul li a {
	white-space: nowrap;
}

div#footer {
    background: url('/static/dark_leather.png') top left repeat;
    margin-top: 30px;
    font-size: 11px;
    font-weight: bold;
    color: #EDEDED;
    padding: 14px 0 0;
    border-top: 5px solid #50adbf;
}

div#footer a {
    color: #5bc4d9;
}

div#footer a:hover {
    color: #EDEDED;
}

div#footer div.ss_copyright {
    margin: 0 auto;
    width: 399px;
    margin-bottom: 10px;
}

div#footer div.favicon_copyright {
    margin: 0 auto;
    width: 393px;
    margin-bottom: 5px;
}

div#footer div.background_copyright {
    margin: 0 auto;
    width: 430px;
    margin-bottom: 15px;
}
